@@ -153,6 +153,40 @@ class CompositeField(collections.Mapping, Field):
         return iter(self._subfields)
 
 
+class ListField(Field):
+    """Base class for fields consisting of a list of several sub-fields."""
+
+    def __init__(self, *args, **kwargs):
+        super(ListField, self).__init__(*args, **kwargs)
+        self._subfields = dict(_collect_fields(self))
+
+    def _load(self, body, resource, nested_in=None):
+        """Load the field list.
+
+        :param body: parent JSON body.
+        :param resource: parent resource.
+        :param nested_in: parent resource name (for error reporting only).
+        :returns: a new list object containing subfields.
+        """
+        nested_in = (nested_in or []) + self._path
+        values = super(ListField, self)._load(body, resource)
+        if values is None:
+            return None
+
+        # Initialize the list that will contain each field instance
+        instances = []
+        for value in values:
+            instance = copy.copy(self)
+            for attr, field in self._subfields.items():
+                # Hide the Field object behind the real value
+                setattr(instance, attr, field._load(value,
+                                                    resource,
+                                                    nested_in))
+            instances.append(instance)
+
+        return instances
+
+
 class MappedField(Field):
     """Field taking real value from a mapping."""
 
@@ -190,13 +224,18 @@ class ResourceBase(object):
         time from here (constructor).
         :param connector: A Connector instance
         :param path: sub-URI path to the resource.
-        :param redfish_version: The version of RedFish. Used to construct
+        :param redfish_version: The version of Redfish. Used to construct
             the object according to schema of the given version.
         """
         self._conn = connector
         self._path = path
         self._json = None
         self.redfish_version = redfish_version
+        # Note(deray): Indicates if the resource holds stale data or not.
+        # Starting off with True and eventually gets set to False when
+        # attribute values are fetched.
+        self._is_stale = True
+
         self.refresh()
 
     def _parse_attributes(self):
@@ -205,20 +244,67 @@ class ResourceBase(object):
             # Hide the Field object behind the real value
             setattr(self, attr, field._load(self.json, self))
 
-    def refresh(self):
+    def refresh(self, force=True):
         """Refresh the resource
 
         Freshly retrieves/fetches the resource attributes and invokes
         ``_parse_attributes()`` method on successful retrieval.
+        It is recommended not to override this method in concrete ResourceBase
+        classes. Resource classes can place their refresh specific operations
+        in ``_do_refresh()`` method, if needed. This method represents the
+        template method in the paradigm of Template design pattern.
+
+        :param force: if set to False, will only refresh if the resource is
+            marked as stale, otherwise neither it nor its subresources will
+            be refreshed.
         :raises: ResourceNotFoundError
         :raises: ConnectionError
         :raises: HTTPError
         """
+        # Note(deray): Don't re-fetch / invalidate the sub-resources if the
+        # resource is "_not_ stale" (i.e. fresh) OR _not_ forced.
+        if not self._is_stale and not force:
+            return
+
         self._json = self._conn.get(path=self._path).json()
         LOG.debug('Received representation of %(type)s %(path)s: %(json)s',
                   {'type': self.__class__.__name__,
                    'path': self._path, 'json': self._json})
         self._parse_attributes()
+        self._do_refresh(force)
+
+        # Mark it fresh
+        self._is_stale = False
+
+    def _do_refresh(self, force):
+        """Primitive method to be overridden by refresh related activities.
+
+        Derived classes are supposed to override this method with the
+        resource specific refresh operations to be performed. This is a
+        primitive method in the paradigm of Template design pattern.
+
+        :param force: should force refresh the resource and its sub-resources,
+            if set to True.
+        :raises: ResourceNotFoundError
+        :raises: ConnectionError
+        :raises: HTTPError
+        """
+
+    def invalidate(self, force_refresh=False):
+        """Mark the resource as stale, prompting refresh() before getting used.
+
+        If ``force_refresh`` is set to True, then it invokes ``refresh()``
+        on the resource.
+
+        :param force_refresh: will invoke refresh on the resource,
+            if set to True.
+        :raises: ResourceNotFoundError
+        :raises: ConnectionError
+        :raises: HTTPError
+        """
+        self._is_stale = True
+        if force_refresh:
+            self.refresh()
 
     @property
     def json(self):
@@ -242,11 +328,11 @@ class ResourceCollectionBase(ResourceBase):
     def __init__(self, connector, path, redfish_version=None):
         """A class representing the base of any Redfish resource collection
 
-        It gets inherited ``ResourceBase`` and invokes the base class
+        It gets inherited from ``ResourceBase`` and invokes the base class
         constructor.
         :param connector: A Connector instance
         :param path: sub-URI path to the resource collection.
-        :param redfish_version: The version of RedFish. Used to construct
+        :param redfish_version: The version of Redfish. Used to construct
             the object according to schema of the given version.
         """
         super(ResourceCollectionBase, self).__init__(connector, path,
